import React, { useState, useEffect } from 'react';
import styles from './DoubleRecord.module.scss';

import TouchOpacity from 'components/TouchOpacity';
import TouchSpan from 'components/TouchSpan';

import Logo from 'assets/img/aia-logo.jpg';
import IconHome from 'assets/img/home.png';
import IconPosition from 'assets/img/position.png';
import IconPositionSelected from 'assets/img/position-selected.png';

import QuestionItem from './DoubleRecord.components/QuestionItem';

const defaultPdfPath = 'https://gitee.com/JYWan/JYWeiBo/raw/master/aia_doc-1.pdf';

// 为什么这个放在useState里面就不能生效，放在外面就可以生效?useState看来也不能用，后面还是用redux吧
let isPlaying = 'N';
const setIsPlaying = (val) => {
  isPlaying = val;
};

export default function(props) {
  // question不一定都是问题，也可能是答案，其实分为需要读/不需要读 和 需要操作/不需要操作
  const defaultSteps = [
    {
      id: '1',
      label: '节点一',
      titleText: '开场',
      questions: [
        {
          questionId: '1',
          needPlay: true,
          needCheck: false,
          playStatus: '0', // 0 未开始；1 播放中；2 暂停中； 3 已结束
          checked: false,
          text: (
            <span>
              <span className={styles.seller}>销售人员：</span>
              根据银保监会有关规定，接下来我将以录音录像的方式对本次销售过程关键环节予以记录，请问您是否同意？
            </span>
          ),
          voice:
            '根据银保监会有关规定，接下来我将以录音录像的方式对本次销售过程关键环节予以记录，请问您是否同意？'
        },
        {
          questionId: '2',
          needPlay: false,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              <span className={styles.buyer}>投保人：</span>
              同意。
            </span>
          ),
          voice: '同意。'
        },
        {
          questionId: '3',
          needPlay: true,
          needCheck: false,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              <span className={styles.seller}>销售人员：</span>
              张三先生，您好，我是友邦保险上海分公司的销售人员02293049423。
            </span>
          ),
          voice: '张三先生，您好，我是友邦保险上海分公司的销售人员02293049423。'
        },
        {
          questionId: '4',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: <span>请销售人员对镜头出示展业证，在镜头上展示3秒，保证对焦准确，画质清晰。</span>,
          voice: '请销售人员对镜头出示展业证，在镜头上展示3秒，保证对焦准确，画质清晰。'
        },
        {
          questionId: '5',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              <span className={styles.seller}>销售人员：</span>
              张三先生，为规范保险销售从业人员的销售行为，也为了更好地保护您的合法权益，请出示您的身份证件，在镜头上展示3秒，保证对焦准确，画质清晰。
            </span>
          ),
          voice:
            ' 张三先生，为规范保险销售从业人员的销售行为，也为了更好地保护您的合法权益，请出示您的身份证件，在镜头上展示3秒，保证对焦准确，画质清晰。'
        }
      ]
    },
    // {
    //   id: '2',
    //   label: '节点二',
    //   titleText: '告知声明',
    //   questions: [
    //     {
    //       questionId: '',
    //       needPlay: false,
    //       needCheck: false,
    //       playStatus: '0',
    //       checked: false,
    //       text: '',
    //       voice: ''
    //     }
    //   ]
    // },
    // {
    //   id: '3',
    //   label: '节点三',
    //   titleText: '确认购买产品关键要素及风险提示（第一部分：投保信息真实性及犹豫期）',
    //   questions: [
    //     {
    //       questionId: '',
    //       needPlay: false,
    //       needCheck: false,
    //       playStatus: '0',
    //       checked: false,
    //       text: '',
    //       voice: ''
    //     }
    //   ]
    // },
    // {
    //   id: '4',
    //   label: '节点四',
    //   titleText: '确认购买产品关键要素及风险提示（第二部分：产品类型及缴费方式）',
    //   questions: [
    //     {
    //       questionId: '',
    //       needPlay: false,
    //       needCheck: false,
    //       playStatus: '0',
    //       checked: false,
    //       text: '',
    //       voice: ''
    //     }
    //   ]
    // },
    // {
    //   id: '5',
    //   label: '节点五',
    //   titleText: '确认购买产品关键要素及风险提示（保险责任、利益不确定性及观察期）',
    //   questions: [
    //     {
    //       questionId: '',
    //       needPlay: false,
    //       needCheck: false,
    //       playStatus: '0',
    //       checked: false,
    //       text: '',
    //       voice: ''
    //     }
    //   ]
    // },
    {
      id: '6',
      label: '节点六',
      titleText:
        '确认购买产品关键要素及风险提示（人身保险投保提示书、计划建议书、免责条款书面说明、风险提示及签字）',
      questions: [
        {
          questionId: '1',
          needPlay: true,
          needCheck: false,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              <span className={styles.seller}>销售人员：</span>
              销售过程中我已向您详细讲解人身保险投保提示书、免除保险人责任条款、保险合同内容变更申请书。为保障您的权益，接下来请您阅读并核实是否与我讲解的内容一致，如您发现任何不一致的地方，请您当场提出。
              接下来，请销售人员向投保人展示相关文件。
            </span>
          ),
          voice:
            '销售过程中我已向您详细讲解人身保险投保提示书、免除保险人责任条款、保险合同内容变更申请书。为保障您的权益，接下来请您阅读并核实是否与我讲解的内容一致，如您发现任何不一致的地方，请您当场提出。接下来，请销售人员向投保人展示相关文件。'
        },
        {
          questionId: '2',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              请销售人员向投保人展示纸质《人身保险投保提示书》，请投保人(张三)认真阅读核实并签字确认，客户签字后，请销售人员向镜头前展示文件名及签字部分，停留至少3秒，保证清晰。
            </span>
          ),
          voice:
            '请销售人员向投保人展示纸质《人身保险投保提示书》，请投保人(张三)认真阅读核实并签字确认，客户签字后，请销售人员向镜头前展示文件名及签字部分，停留至少3秒，保证清晰。'
        },
        {
          questionId: '3',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>请销售人员点击链接向镜头清晰展示纸质《人身保险投保提示书》标题及签字部分。</span>
          ),
          voice: '请销售人员点击链接向镜头清晰展示纸质《人身保险投保提示书》标题及签字部分。'
        },
        {
          questionId: '4',
          needPlay: true,
          needCheck: false,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              请销售人员向投保人展示免除保险人责任条款说明，请投保人特别关注条款中的“明确说明与如实告知、年龄计算及错误处理、保险事故的通知等内容”。请投保人认真阅读核实，稍后请投保人签字确认。
            </span>
          ),
          voice:
            '请销售人员向投保人展示免除保险人责任条款说明，请投保人特别关注条款中的“明确说明与如实告知、年龄计算及错误处理、保险事故的通知等内容”。请投保人认真阅读核实，稍后请投保人签字确认。'
        },
        {
          questionId: '5',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              请销售人员点击链接向镜头展示
              <TouchSpan
                className={styles.extlink}
                onClick={() => {
                  showPDF(defaultPdfPath, '免除保险人责任条款');
                }}
              >
                《免除保险人责任条款》
              </TouchSpan>
              至少3秒，保证标题清晰后给投保人阅读。
            </span>
          ),
          voice:
            '请销售人员点击链接向镜头展示《免除保险人责任条款》至少3秒，保证标题清晰后给投保人阅读。'
        },
        {
          questionId: '6',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: <span>请销售人员确认已向镜头清晰展示免除保险人责任条款。</span>,
          voice: '请销售人员确认已向镜头清晰展示免除保险人责任条款。'
        },
        {
          questionId: '7',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: <span>请销售人员向投保人展示保险条款，请投保人认真阅读。</span>,
          voice: '请销售人员向投保人展示保险条款，请投保人认真阅读。'
        },
        {
          questionId: '8',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              请销售人员点击链接向镜头展示
              <TouchSpan
                className={styles.extlink}
                onClick={() => {
                  showPDF(defaultPdfPath, '保险条款');
                }}
              >
                《保险条款》
              </TouchSpan>
              至少3秒，保证标题清晰后给投保人阅读
            </span>
          ),
          voice: '请销售人员点击链接向镜头展示《保险条款》至少3秒，保证标题清晰后给投保人阅读。'
        },
        {
          questionId: '9',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: <span>请销售人员确认已向镜头清晰展示保险条款。</span>,
          voice: '请销售人员确认已向镜头清晰展示保险条款。'
        },
        {
          questionId: '10',
          needPlay: true,
          needCheck: false,
          playStatus: '0',
          checked: false,
          text: <span>请销售人员向投保人展示保险合同内容变更申请书，请投保人认真阅读。</span>,
          voice: '请销售人员向投保人展示保险合同内容变更申请书，请投保人认真阅读。'
        },
        {
          questionId: '11',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: (
            <span>
              请销售人员点击链接向镜头展示
              <TouchSpan
                className={styles.extlink}
                onClick={() => {
                  showPDF(defaultPdfPath, '保险合同内容变更申请书');
                }}
              >
                保险合同内容变更申请书
              </TouchSpan>
              至少3秒，保证标题清晰后给投保人阅读。稍后请投保人签字确认。
            </span>
          ),
          voice:
            '请销售人员点击链接向镜头展示 保险合同内容变更申请书 至少3秒，保证标题清晰后给投保人阅读。稍后请投保人签字确认。'
        },
        {
          questionId: '12',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: <span>请销售人员确认已向镜头清晰展示保险合同内容变更申请书。</span>,
          voice: '请销售人员确认已向镜头清晰展示保险合同内容变更申请书。'
        }
      ]
    },
    {
      id: '7',
      label: '节点七',
      titleText: '签字确认',
      questions: [
        {
          questionId: '1',
          needPlay: true,
          needCheck: false,
          playStatus: '0',
          checked: false,
          text:
            '刚才已请投保人阅读过投保提示书、免除保险人责任条款、保险条款及保险合同内容变更申请书，在确保镜头可以录制客户签字过程情况下，请投保人张三签字确认。',
          voice:
            '刚才已请投保人阅读过投保提示书、免除保险人责任条款、保险条款及保险合同内容变更申请书，在确保镜头可以录制客户签字过程情况下，请投保人张三签字确认。'
        },
        {
          questionId: '2',
          needPlay: true,
          needCheck: true,
          playStatus: '0',
          checked: false,
          text: '客户签署完成后，请销售人员向镜头展示客户签字至少3秒，保证清晰。',
          voice: '客户签署完成后，请销售人员向镜头展示客户签字至少3秒，保证清晰。'
        }
      ]
    }
  ];

  const [steps, updateSteps] = useState(defaultSteps);
  const [imgSrc1, setImgSrc1] = useState('');
  const [imgSrc2, setImgSrc2] = useState('');
  const [stepId, setStepId] = useState('1');
  const [questionId, setQuestionId] = useState('1');
  // const [isPlaying, setIsPlaying] = useState('N');

  const currentStep = steps.find((s) => s.id === stepId);

  useEffect(() => {
    // console.log('use effect');
    setTimeout(() => {
      playQuestion(questionId);
    }, 1000);
  }, [stepId, questionId]);

  // 朗读当前问题
  const playQuestion = (questionId) => {
    console.log('407');
    const targetQuestion = currentStep.questions.find((q) => q.questionId === questionId);
    if (!targetQuestion || targetQuestion.playStatus === '3') {
      return;
    } else if (!targetQuestion.needPlay) {
      setPlayStatus(questionId, '3');
    } else if(targetQuestion.playStatus !== '2'){
      setPlayStatus(questionId, '1');
      voiceStart(targetQuestion.voice);
    }
  };

  // 设置问题的播放状态，以展示不同的图标
  const setPlayStatus = (qId, playStatus) => {
    const newSteps = steps.map((stp) => ({
      ...stp,
      questions: stp.questions.map((qst) => ({
        ...qst
      }))
    }));
    const stepIdx = newSteps.findIndex((s) => s.id === stepId);
    const questionIdx = currentStep.questions.findIndex((q) => q.questionId === qId);

    if (stepIdx === -1 || questionIdx === -1) {
      console.log('error question');
      return;
    }

    newSteps[stepIdx].questions[questionIdx].playStatus = playStatus;
    updateSteps(newSteps);

    // console.log(`设置问题${qId}为${playStatus}`);
  };

  // 勾选，用于需要勾选的问题
  const setChecked = (qId, checked) => {
    const newSteps = steps.map((stp) => ({
      ...stp,
      questions: stp.questions.map((qst) => ({
        ...qst
      }))
    }));
    const stepIdx = newSteps.findIndex((s) => s.id === stepId);
    const questionIdx = currentStep.questions.findIndex((q) => q.questionId === qId);

    if (stepIdx === -1 || questionIdx === -1) {
      console.log('error question');
      return;
    }

    newSteps[stepIdx].questions[questionIdx].checked = checked;
    updateSteps(newSteps);

    // console.log(checked);
  };

  // 勾选，然后去下一个问题
  const setCheckThenNextQuestion = () => {
    setChecked(questionId, true);
    goNextQuestion();
  };

  // 同时修改两个state,应该有更好的办法，不用写在一个方法里面
  const setPlayStatusAndChecked = (qId, playStatus, checked) => {
    const newSteps = steps.map((stp) => ({
      ...stp,
      questions: stp.questions.map((qst) => ({
        ...qst
      }))
    }));
    const stepIdx = newSteps.findIndex((s) => s.id === stepId);
    const questionIdx = currentStep.questions.findIndex((q) => q.questionId === qId);

    if (stepIdx === -1 || questionIdx === -1) {
      console.log('error question');
      return;
    }

    newSteps[stepIdx].questions[questionIdx].playStatus = playStatus;
    newSteps[stepIdx].questions[questionIdx].checked = checked;
    updateSteps(newSteps);

    // console.log(checked);
  };

  // app结束朗读后调用
  const finishOneVoice = () => {
    const targetQuestion = currentStep.questions.find((q) => q.questionId === questionId);

    if (targetQuestion.needCheck) {
      // stop and wait check
      setPlayStatus(questionId, '3');
      // console.log('need you check');
      return;
    } else {
      setPlayStatusAndChecked(questionId, '3', true);
      goNextQuestion();
    }
  };

  // 下一个问题
  const goNextQuestion = () => {
    const newSteps = steps.map((stp) => ({
      ...stp,
      questions: stp.questions.map((qst) => ({
        ...qst
      }))
    }));
    const stepIdx = newSteps.findIndex((s) => s.id === stepId);
    const questionIdx = currentStep.questions.findIndex((q) => q.questionId === questionId);

    if (questionIdx === currentStep.questions.length - 1) {
      console.log('this is the last question');
      return;
    }

    if (stepIdx === -1 || questionIdx === -1) {
      console.log('error question');
      return;
    }

    const nextQuestionId = newSteps[stepIdx].questions[questionIdx + 1].questionId;

    setQuestionId(nextQuestionId); // 设置完之后触发useEffect,由useEffect调用app的朗读
  };

  // 调用app的展示PDF方法，合同展示节点使用
  const showPDF = (pdfPath, pdfName) => {
    console.log('show pdf');
    if (isPlaying === 'Y') {
      return;
    }
    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }
    window.webkit.messageHandlers.aiaPDFPreView.postMessage({
      pdfPath,
      pdfName
    });
  };

  // 调用app的语音播报，监听语音结束，在刚进入页面时自动调用一次，每一个问题结束后判断是否自动调用
  const voiceStart = (voiceContent) => {
    setIsPlaying('Y');
    console.log(`播放: ${voiceContent}`);

    // setTimeout(() => {
    //   console.log('finish one voice');
    //   finishOneVoice();
    // }, 3000);

    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }
    window.webkit.messageHandlers.aiaVoiceBroadCastContent.postMessage({ content: voiceContent });

    window.aiaVoiceFinish = () => {
      console.log('finish one step');
      setIsPlaying('N');
      finishOneVoice();
    };
  };

  // 调用app的语音暂停，点击问题的暂停按钮时候调用
  const voicePause = () => {
    console.log('暂停播放');
    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }
    window.webkit.messageHandlers.aiaVoiceBroadPause.postMessage({});
  };

  // 调用app的暂停后继续播放方法
  const voiceContinue = () => {
    console.log('继续播放');
    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }
    window.webkit.messageHandlers.aiaVoiceBroadContinuePlay.postMessage({});
  };

  // 调用app的语音停止，在跳转、翻页的时候调用，避免翻页后继续播放
  const voiceStop = () => {
    console.log('停止播放');
    setIsPlaying('N');
    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }
    window.webkit.messageHandlers.aiaVoiceBroadStop.postMessage({});
  };

  // 调用app的签名方法，展示签名组件，签名节点使用
  const showSign1 = () => {
    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }

    window.webkit.messageHandlers.aiaShowSignView.postMessage({});
    window.aiaGetSignImage = (base64Data) => {
      const decodeBase64 = decodeURIComponent(base64Data);
      const base64Src = 'data:image/png;base64,' + decodeBase64;
      setImgSrc1(base64Src);
    };
  };

  // 调用app的签名方法，展示签名组件，签名节点使用
  const showSign2 = () => {
    if (!window.webkit) {
      alert('window.webkit 未定义');
      return;
    }

    window.webkit.messageHandlers.aiaShowSignView.postMessage({});
    window.aiaGetSignImage = (base64Data) => {
      const decodeBase64 = decodeURIComponent(base64Data);
      const base64Src = 'data:image/png;base64,' + decodeBase64;
      setImgSrc2(base64Src);
    };
  };

  // 测试用，在图片获取后查看图片src
  const showImgSrc = () => {
    // alert(
    //   JSON.stringify({
    //     imgSrc1
    //   })
    // );
  };

  // 查找最后一个未读的问题，用于跳转上一步、下一步之后回到刚才的问题。
  const findLastUnreadQuestion = (sId) => {
    const questions = steps.find((s) => s.id === sId).questions;
    const len = questions.length;
    let questionId = '';
    for (let i = 0; i < len; i++) {
      questionId = questions[i].questionId;
      if (!questions[i].checked) {
        break;
      }
    }
    return questionId;
  };

  // 上一个节点，如果是第一节点则回到上一页
  const lastStep = () => {
    if (stepId === '1') {
      voiceStop();
      setPlayStatus(questionId, '3');
      window.history.go(-1);
    } else {
      const currentStepIdx = steps.findIndex((s) => s.id === stepId);
      const lastStepId = steps[currentStepIdx - 1].id;

      voicePause();
      setPlayStatus(questionId, '2');
      setStepId(lastStepId);
      setQuestionId(findLastUnreadQuestion(lastStepId));
    }
  };

  // 下一个节点，如果是最后节点则前往保户服务页面
  const nextStep = () => {
    if (currentStep.questions.some((q) => !q.checked)) {
      alert('当前节点问题尚未全部确认');
      return;
    }
    setPlayStatus(questionId, '3');
    // voiceStop();
    if (stepId === 7) {
      // setSignVisible(true);
      return;
    } else {
      const currentStepIdx = steps.findIndex((s) => s.id === stepId);
      const nextStepId = steps[currentStepIdx + 1].id;

      setStepId(nextStepId);
      setQuestionId(findLastUnreadQuestion(nextStepId));
    }
  };

  // 前往某个节点 暂时不使用
  // const toStep = (sId) => {
  //   setStepId(sId);
  //   setQuestionId('1');
  // };

  return (
    <div className={styles.page}>
      <div className={styles.header}>
        <img
          src={Logo}
          alt="logo"
          className={styles.aiaLogo}
          onClick={() => {
            console.log(JSON.stringify({ stepId, questionId, isPlaying }));
          }}
        ></img>
        <div>双录指引</div>
        <div className={styles.homeIcon}>
          <img src={IconHome} alt="logo"></img>
        </div>
      </div>
      <div className={styles.main}>
        <div className={styles.stepArea}>
          {steps.map((item) => (
            <div
              key={item.id}
              className={stepId === item.id ? styles.stepIndexSelected : styles.stepIndex}
            >
              <img
                src={stepId === item.id ? IconPositionSelected : IconPosition}
                alt="icon-position"
              ></img>
              <span>{item.label}</span>
            </div>
          ))}
        </div>
        <div className={styles.questionArea}>
          <div className={styles.stepTitle}>{currentStep.titleText}</div>
          <div className={styles.stepQuestions}>
            {currentStep.questions.map((q) => (
              <QuestionItem
                key={q.questionId}
                question={q}
                setPlayStatus={setPlayStatus}
                setChecked={setCheckThenNextQuestion}
                voiceStart={voiceStart}
                voicePause={voicePause}
                voiceContinue={voiceContinue}
              />
            ))}
            {stepId === '7' && (
              <>
                <TouchOpacity className={styles.btn} onClick={showSign1}>
                  投保人/被保人签名
                </TouchOpacity>
                <div style={{ clear: 'both' }}></div>
                <div>签名图片展示:</div>
                <div onClick={showImgSrc}>
                  <img alt="签名图片" src={imgSrc1}></img>
                </div>

                <TouchOpacity className={styles.btn} onClick={showSign2}>
                  营销员签名
                </TouchOpacity>
                <div style={{ clear: 'both' }}></div>
                <div>签名图片展示:</div>
                <div onClick={showImgSrc}>
                  <img alt="签名图片" src={imgSrc2}></img>
                </div>
              </>
            )}
          </div>
        </div>
      </div>
      <div className={styles.footer}>
        <TouchOpacity className={styles.footerBtn} onClick={lastStep}>
          上一步
        </TouchOpacity>
        <TouchOpacity className={styles.footerBtn} onClick={nextStep}>
          下一步
        </TouchOpacity>
      </div>
    </div>
  );
}
