import React, { useState } from 'react';
import { connect } from 'react-redux';
import { Table, Divider } from 'antd';
import 'antd/dist/antd.css';

import styles from './InsuredInfo.module.scss';
import IconAIA from '../assets/img/aia-icon.jpg';
import IconDetail from '../assets/img/detail.png';
import IconCancel from '../assets/img/cancel.png';
import IconCalculate from '../assets/img/calculate.png';
import IconCustomer from 'assets/img/customer.png';
import IconFile from 'assets/img/file.png';
import IconMoney from 'assets/img/money.png';
import IconReturn from 'assets/img/return.png';
import IconHome from 'assets/img/home.png';

import TouchOpacity from '../components/TouchOpacity';
import TouchSpan from '../components/TouchSpan';
import TabTitle from './InsuredsInfo.components/TabTitle';
import ProductListModal from './InsuredsInfo.components/ProductListModal';

const initTableData = [
  {
    key: '1',
    plan: '天佑一生”五合一”疾病保险',
    insured: '16516007',
    invalidDate: '2020-02-01',
    status: '付费期间',
    amount: '18000'
  },
  {
    key: '2',
    plan: '附加惠添益恶性肿瘤疾病保险',
    insured: '16516007',
    invalidDate: '2020-02-01',
    status: '付费期间',
    amount: '9000'
  }
];

const sleep = (ms) => new Promise((resolve) => setTimeout(resolve, ms));

const InsuredInfo = (props) => {
  const { history, showLoading, hideLoading } = props;
  const [selectedDataKey, setSelectedDataKey] = useState('');
  const [tableData, setTableData] = useState(initTableData);
  const [modalVisible, setModalVisible] = useState(false);

  const columns = [
    {
      title: '保险计划',
      dataIndex: 'plan',
      key: 'plan',
      render: (text) => <span>{text}</span>
    },
    {
      title: '被保险人',
      dataIndex: 'insured',
      key: 'insured'
    },
    {
      title: '合同生效日',
      dataIndex: 'invalidDate',
      key: 'invalidDate'
    },
    {
      title: '合同状态',
      dataIndex: 'status',
      key: 'status'
    },
    {
      title: '基本保额',
      dataIndex: 'amount',
      key: 'amount'
    },
    {
      title: '操作',
      key: 'action',
      render: (text, record) => (
        <div>
          <TouchSpan className={styles.actText}>修改保额</TouchSpan>
          <Divider type="vertical" />
          <TouchSpan
            className={record.key === '1' ? styles.actTextDefault : styles.actText}
            onClick={() => {
              removeTableData(record.key);
            }}
          >
            删除
          </TouchSpan>
        </div>
      )
    }
  ];

  const addPolicy = () => {
    setModalVisible(!modalVisible);
  };

  const removeTableData = async (key) => {
    if (key === '1') {
      return;
    }

    showLoading();
    await sleep(500);
    hideLoading();

    const newTableData = tableData.filter((ele) => ele.key !== key);
    setTableData(newTableData);
  };

  const addTableData = async (product) => {
    const key = `add-${product.id}`;
    if (tableData.some((t) => t.key === key)) {
      alert('该产品已添加过，请勿重复添加');
      return;
    }

    showLoading();
    await sleep(500);
    hideLoading();

    setSelectedDataKey(key);
    setTableData([
      ...tableData,
      {
        key,
        plan: product.name,
        insured: '16516007',
        invalidDate: '2020-02-01',
        status: '付费期间',
        amount: '9000'
      }
    ]);
  };

  const setRowClassName = (record) => {
    return record.key === selectedDataKey ? styles.highLightLine : '';
  };

  const submit = () => {
    history.push('/doubleRecord');
  };

  return (
    <div className={styles.page}>
      <div className={styles.header}>
        <img alt="logo" src={IconAIA} className={styles.headerImg} />
        <TabTitle>
          <img alt="customer" src={IconCustomer} className={styles.titleIcon} />
          <span>客户管理</span>
        </TabTitle>
        <TabTitle>
          <img alt="policy" src={IconFile} className={styles.titleIcon} />
          <span>保单信息</span>
        </TabTitle>
        <TabTitle>
          <img alt="pay" src={IconMoney} className={styles.titleIcon} />
          <span>借/领/退款</span>
        </TabTitle>
        <div className={styles.user}>
          <div>
            <img src={IconReturn} alt="return"></img>
            <img src={IconHome} alt="home"></img>
          </div>
          <span>C161361507</span>
        </div>
      </div>
      <div className={styles.headerExt}></div>
      <div className={styles.main}>
        <div className={styles.leftMenu}>
          <div className={styles.leftMenuItem}>保障计划</div>
          <div className={styles.leftMenuItem}>合同权益</div>
          <div className={styles.leftMenuItem}>银行账户</div>
        </div>
        <div className={styles.content}>
          <div>
            <span className={styles.descText}>保费到期日: 2020年10月31日</span>
            <span className={styles.descText}>支付方式:月付</span>
          </div>
          <div className={styles.btns}>
            <TouchOpacity onClick={addPolicy} className={styles.btn}>
              增加附加合同
            </TouchOpacity>
            <TouchOpacity className={styles.btn}>变更支付方式</TouchOpacity>
            <TouchOpacity className={styles.btn}>复效/垫缴不足一期还款</TouchOpacity>
            <TouchOpacity className={styles.btn}>智选康康产品转换</TouchOpacity>
          </div>
          <div className={styles.tableBox}>
            <Table
              columns={columns}
              dataSource={tableData}
              pagination={false}
              rowClassName={setRowClassName}
            />
          </div>
          <div className={styles.accountLine}>
            <span className={styles.actText}>已签约授权</span>
            <span>付款账号：622********3042</span>
          </div>
          <div className={styles.premLine}>
            <span>原保费：860元</span>
            <span>
              下一期保费：<span className={styles.actText}>{tableData.length * 180 + 500}元</span>
            </span>
            <TouchOpacity className={styles.actText}>
              <img src={IconDetail} alt="detail"></img>
              款项明细
            </TouchOpacity>
          </div>
          <div className={styles.actionLine}>
            <TouchOpacity className={styles.actText}>
              <img src={IconCancel} alt="cancel"></img>
              撤销
            </TouchOpacity>
            <TouchOpacity className={styles.actText}>
              <img src={IconCalculate} alt="calculate"></img>
              保费试算
            </TouchOpacity>
          </div>
        </div>
      </div>
      <div className={styles.footer}>
        <TouchOpacity className={styles.footerBtn} onClick={submit}>
          提交
        </TouchOpacity>
      </div>

      <ProductListModal
        visible={modalVisible}
        addTableData={addTableData}
        showLoading={showLoading}
        hideLoading={hideLoading}
        close={() => {
          setModalVisible(false);
        }}
      />
    </div>
  );
};

const mapDispatch = ({ app: { showLoading, hideLoading } }) => ({
  showLoading,
  hideLoading
});
export default connect(null, mapDispatch)(InsuredInfo);
